@@ -18,6 +18,8 @@ Classes to represent entities in a tree.
 
 from PySide2.QtCore import Qt
 from PySide2.QtGui import QFont, QBrush, QIcon
+
+from spinetoolbox.helpers import DB_ITEM_SEPARATOR
 from .multi_db_tree_item import MultiDBTreeItem
 
 
@@ -27,7 +29,7 @@ class EntityRootItem(MultiDBTreeItem):
 
     @property
     def display_id(self):
-        """"See super class."""
+        """ "See super class."""
         return "root"
 
     @property
@@ -36,7 +38,7 @@ class EntityRootItem(MultiDBTreeItem):
 
     @property
     def display_data(self):
-        """"See super class."""
+        """ "See super class."""
         return "root"
 
     def set_data(self, column, value, role):
@@ -97,34 +99,6 @@ class EntityClassItem(MultiDBTreeItem):
                 return QBrush(Qt.gray)
         return super().data(column, role)
 
-    def raise_group_children_by_id(self, db_map_ids):
-        """
-        Moves group children to the top of the list.
-
-        Args:
-            db_map_ids (dict): set of ids corresponding to newly inserted group children,
-                keyed by DiffDatabaseMapping
-        """
-        rows = set(row for db_map, ids in db_map_ids.items() for row in self.find_rows_by_id(db_map, *ids))
-        self._raise_group_children_by_row(rows)
-
-    def _raise_group_children_by_row(self, rows):
-        """
-        Moves group children to the top of the list.
-
-        Args:
-            rows (set, list): collection of rows corresponding to newly inserted group children
-        """
-        rows = [row for row in rows if row >= self._group_child_count]
-        if not rows:
-            return
-        self.model.layoutAboutToBeChanged.emit()
-        group_children = list(reversed([self.children.pop(row) for row in sorted(rows, reverse=True)]))
-        self.insert_children(self._group_child_count, group_children)
-        self.model.layoutChanged.emit()
-        self._group_child_count += len(group_children)
-        self._refresh_child_map()
-
     def remove_children(self, position, count):
         """
         Overriden method to keep the group child count up to date.
@@ -138,8 +112,8 @@ class EntityClassItem(MultiDBTreeItem):
             self._group_child_count -= removed_child_count
         return True
 
-    def fetch_successful(self, db_map, item):
-        return item["class_id"] == self.db_map_id(db_map)
+    def filter_query(self, query, subquery, db_map):
+        return query.filter(subquery.c.class_id == self.db_map_id(db_map))
 
     def set_data(self, column, value, role):
         """See base class."""
@@ -160,6 +134,11 @@ class ObjectClassItem(EntityClassItem):
         """Return data to put as default in a parameter table when this item is selected."""
         return dict(object_class_name=self.display_data, database=self.first_db_map.codename)
 
+    @property
+    def _children_sort_key(self):
+        """Reimplemented so groups are above non-groups."""
+        return lambda item: (not item.is_group(), item.display_id)
+
 
 class RelationshipClassItem(EntityClassItem):
     """A relationship_class item."""
@@ -182,11 +161,11 @@ class ObjectRelationshipClassItem(RelationshipClassItem):
         """See base class."""
         return False
 
-    def fetch_successful(self, db_map, item):
+    def filter_query(self, query, subquery, db_map):
         object_id = self.parent_item.db_map_id(db_map)
-        return super().fetch_successful(db_map, item) and object_id in {
-            int(id_) for id_ in item["object_id_list"].split(",")
-        }
+        ids = set(x.id for x in db_map.query(db_map.relationship_sq).filter_by(object_id=object_id))
+        query = query.filter(db_map.in_(subquery.c.id, ids))
+        return super().filter_query(query, subquery, db_map)
 
 
 class MemberObjectClassItem(ObjectClassItem):
@@ -194,12 +173,9 @@ class MemberObjectClassItem(ObjectClassItem):
 
     item_type = "members"
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-
     @property
     def display_id(self):
-        # Return an empty tuple so we never insert anything before this item (see _insert_children_sorted)
+        # Return an empty tuple so we never insert anything above this item (see _insert_children_sorted)
         return ()
 
     @property
@@ -217,8 +193,9 @@ class MemberObjectClassItem(ObjectClassItem):
             self.first_db_map, super().item_type, self.db_map_id(self.first_db_map), for_group=False
         )
 
-    def fetch_successful(self, db_map, item):
-        return item["class_id"] == self.db_map_id(db_map) and item["group_id"] == self.parent_item.db_map_id(db_map)
+    def filter_query(self, query, subquery, db_map):
+        query = query.filter(subquery.c.group_id == self.parent_item.db_map_id(db_map))
+        return super().filter_query(query, subquery, db_map)
 
     @property
     def child_item_class(self):
@@ -241,14 +218,13 @@ class MemberObjectClassItem(ObjectClassItem):
 class EntityItem(MultiDBTreeItem):
     """An entity item."""
 
-    _has_members_item = False
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._has_members_item = False
 
     @property
     def members_item(self):
-        if not self._has_members_item:
-            # Insert members item. Note that we pass the db_map_ids of the parent object class item
-            self.insert_children(0, [MemberObjectClassItem(self.model, self.parent_item.db_map_ids.copy())])
-            self._has_members_item = True
+        self._fetch_members_item()
         return self.child(0)
 
     @property
@@ -256,15 +232,8 @@ class EntityItem(MultiDBTreeItem):
         """Returns corresponding class icon."""
         return self.parent_item._display_icon(for_group=self.is_group())
 
-    def db_map_member_ids(self, db_map):
-        return [x["member_id"] for x in self.db_map_entity_groups(db_map)]
-
-    def db_map_entity_groups(self, db_map):
-        # FIXME: We might need to fetch here
-        return self.db_mngr.get_items_by_field(db_map, "entity_group", "group_id", self.db_map_id(db_map))
-
     def is_group(self):
-        return self.members_item.has_children()
+        return any(self.db_map_data_field(db_map, "group_id") is not None for db_map in self.db_maps)
 
     def data(self, column, role=Qt.DisplayRole):
         if role == Qt.ToolTipRole:
@@ -275,6 +244,22 @@ class EntityItem(MultiDBTreeItem):
         """See base class."""
         return False
 
+    def _can_fetch_members_item(self):
+        return self.is_group() and not self._has_members_item
+
+    def _fetch_members_item(self):
+        if self._can_fetch_members_item():
+            # Insert members item. Note that we pass the db_map_ids of the parent object class item
+            self.insert_children(0, [MemberObjectClassItem(self.model, self.parent_item.db_map_ids.copy())])
+            self._has_members_item = True
+
+    def can_fetch_more(self):
+        return super().can_fetch_more() or self._can_fetch_members_item()
+
+    def fetch_more(self):
+        super().fetch_more()
+        self._fetch_members_item()
+
 
 class ObjectItem(EntityItem):
     """An object item."""
@@ -294,9 +279,10 @@ class ObjectItem(EntityItem):
             database=self.first_db_map.codename,
         )
 
-    def fetch_successful(self, db_map, item):
+    def filter_query(self, query, subquery, db_map):
         object_class_id = self.db_map_data_field(db_map, 'class_id')
-        return object_class_id in {int(id_) for id_ in item["object_class_id_list"].split(",")}
+        ids = set(x.id for x in db_map.query(db_map.relationship_class_sq).filter_by(object_class_id=object_class_id))
+        return query.filter(db_map.in_(subquery.c.id, ids))
 
 
 class MemberObjectItem(ObjectItem):
@@ -311,7 +297,7 @@ class MemberObjectItem(ObjectItem):
 
     @property
     def display_data(self):
-        """"Returns the name for display."""
+        """ "Returns the name for display."""
         return self.db_map_data_field(self.first_db_map, "member_name")
 
     def has_children(self):
@@ -338,8 +324,8 @@ class RelationshipItem(EntityItem):
 
     @property
     def display_data(self):
-        """"Returns the name for display."""
-        return self.db_mngr.GROUP_SEP.join(
+        """ "Returns the name for display."""
+        return DB_ITEM_SEPARATOR.join(
             [x for x in self.object_name_list.split(",") if x != self.parent_item.parent_item.display_data]
         )
 
