@@ -16,9 +16,10 @@ Provides pivot table models for the Tabular View.
 :date:   1.11.2018
 """
 
-from PySide2.QtCore import Qt, Slot, QTimer, QAbstractTableModel, QModelIndex, QSortFilterProxyModel
+from PySide2.QtCore import Qt, Signal, Slot, QTimer, QAbstractTableModel, QModelIndex, QSortFilterProxyModel
 from PySide2.QtGui import QColor, QFont
 from spinedb_api.parameter_value import join_value_and_type, split_value_and_type
+from spinetoolbox.helpers import DB_ITEM_SEPARATOR, parameter_identifier
 from .pivot_model import PivotModel
 from ...mvcmodels.shared import PARSED_ROLE
 from ...config import PIVOT_TABLE_HEADER_COLOR
@@ -27,6 +28,35 @@ from ..widgets.custom_delegates import (
     ParameterPivotTableDelegate,
     ScenarioAlternativeTableDelegate,
 )
+from ...helpers import ItemTypeFetchParent
+
+
+class _ParameterFetchParent(ItemTypeFetchParent):
+    def __init__(self, fetch_item_type, parent):
+        super().__init__(fetch_item_type)
+        self._parent = parent
+
+    def filter_query(self, query, subquery, db_map):
+        return query.filter(subquery.c.entity_class_id == self._parent.current_class_id[db_map])
+
+
+class _EntityFetchParent(ItemTypeFetchParent):
+    def __init__(self, fetch_item_type, parent):
+        super().__init__(fetch_item_type)
+        self._parent = parent
+
+    def filter_query(self, query, subquery, db_map):
+        return query.filter(subquery.c.class_id == self._parent.current_class_id[db_map])
+
+
+class _MemberObjectFetchParent(ItemTypeFetchParent):
+    def __init__(self, fetch_item_type, parent):
+        super().__init__(fetch_item_type)
+        self._parent = parent
+
+    def filter_query(self, query, subquery, db_map):
+        object_class_id_list = {x[db_map] for x in self._parent.current_object_class_id_list}
+        return query.filter(db_map.in_(subquery.c.class_id, object_class_id_list))
 
 
 class PivotTableModelBase(QAbstractTableModel):
@@ -35,12 +65,14 @@ class PivotTableModelBase(QAbstractTableModel):
     _MAX_FETCH_COUNT = 1000
     _FETCH_DELAY = 0
 
+    model_data_changed = Signal()
+
     def __init__(self, parent):
         """
         Args:
             parent (SpineDBEditor)
         """
-        super().__init__()
+        super().__init__(parent)
         self._parent = parent
         self.db_mngr = parent.db_mngr
         self.model = PivotModel()
@@ -53,15 +85,27 @@ class PivotTableModelBase(QAbstractTableModel):
         self.modelAboutToBeReset.connect(self.reset_data_count)
         self.modelReset.connect(lambda *args: QTimer.singleShot(self._FETCH_DELAY, self.start_fetching))
 
-    def canFetchMore(self, parent=QModelIndex()):
-        return any(self.db_mngr.can_fetch_more(db_map, self.item_type, parent=self) for db_map in self._parent.db_maps)
+    def _fetch_parents(self):
+        """Yields fetch parents for this model.
 
-    def fetchMore(self, parent=QModelIndex()):
+        Yields:
+            FetchParent
+        """
+        raise NotImplementedError()
+
+    def _can_fetch_more_parent(self, parent):
+        return any(self.db_mngr.can_fetch_more(db_map, parent) for db_map in self._parent.db_maps)
+
+    def canFetchMore(self, _parent):
+        return any(self._can_fetch_more_parent(parent) for parent in self._fetch_parents())
+
+    def _fetch_more_parent(self, parent):
         for db_map in self._parent.db_maps:
-            self.db_mngr.fetch_more(db_map, self.item_type, parent=self)
+            self.db_mngr.fetch_more(db_map, parent)
 
-    def fetch_successful(self, db_map, item):
-        return True
+    def fetchMore(self, _parent):
+        for parent in self._fetch_parents():
+            self._fetch_more_parent(parent)
 
     @property
     def item_type(self):
@@ -153,6 +197,7 @@ class PivotTableModelBase(QAbstractTableModel):
         top_left = self.index(self.headerRowCount(), self.headerColumnCount())
         bottom_right = self.index(self.rowCount() - 1, self.columnCount() - 1)
         self.dataChanged.emit(top_left, bottom_right)
+        self.model_data_changed.emit()
 
     def remove_from_model(self, data):
         if not data:
@@ -194,11 +239,11 @@ class PivotTableModelBase(QAbstractTableModel):
 
     def headerRowCount(self):
         """Returns number of rows occupied by header."""
-        return len(self.model.pivot_columns) + bool(self.model.pivot_rows)
+        return len(self.model.pivot_columns) + int(bool(self.model.pivot_rows))
 
     def headerColumnCount(self):
         """Returns number of columns occupied by header."""
-        return max(bool(self.model.pivot_columns), len(self.model.pivot_rows))
+        return max(int(bool(self.model.pivot_columns)), len(self.model.pivot_rows))
 
     def dataRowCount(self):
         """Returns number of rows that contain actual data."""
@@ -305,12 +350,11 @@ class PivotTableModelBase(QAbstractTableModel):
         return False
 
     def headerData(self, section, orientation, role=Qt.DisplayRole):
-        if role == Qt.DisplayRole and orientation == Qt.Horizontal:
-            if section == self._plot_x_column:
+        if role == Qt.DisplayRole:
+            if orientation == Qt.Horizontal and section == self._plot_x_column:
                 return "(X)"
-            return None
-        if role == Qt.DisplayRole and orientation == Qt.Vertical:
-            return self._V_HEADER_WIDTH * " "
+            if orientation == Qt.Vertical:
+                return self._V_HEADER_WIDTH * " "
         return None
 
     def map_to_pivot(self, index):
@@ -708,15 +752,24 @@ class ParameterValuePivotTableModel(PivotTableModelBase):
             parent (SpineDBEditor)
         """
         super().__init__(parent)
-        self._object_class_count = None
+        self._object_fetch_parent = _EntityFetchParent("object", self._parent)
+        self._relationship_fetch_parent = _EntityFetchParent("relationship", self._parent)
+        self._parameter_value_fetch_parent = _ParameterFetchParent("parameter_value", self._parent)
+        self._parameter_definition_fetch_parent = _ParameterFetchParent("parameter_definition", self._parent)
+        self._alternative_fetch_parent = ItemTypeFetchParent("alternative")
 
     @property
     def item_type(self):
         return "parameter_value"
 
-    def fetch_successful(self, db_map, item):
-        entity_class_id = self._parent.current_class_id[db_map]
-        return item["entity_class_id"] == entity_class_id
+    def _fetch_parents(self):
+        yield self._parameter_value_fetch_parent
+        yield self._alternative_fetch_parent
+        yield self._parameter_definition_fetch_parent
+        if self._parent.current_class_type == "object_class":
+            yield self._object_fetch_parent
+        elif self._parent.current_class_type == "relationship_class":
+            yield self._relationship_fetch_parent
 
     def db_map_object_ids(self, index):
         """
@@ -771,15 +824,7 @@ class ParameterValuePivotTableModel(PivotTableModelBase):
         if not self.index_in_data(index):
             return ""
         object_names, parameter_name, alternative_name, db_name = self._all_header_names(index)
-        return (
-            self.db_mngr.GROUP_SEP.join(object_names)
-            + " - "
-            + parameter_name
-            + " - "
-            + alternative_name
-            + " - "
-            + db_name
-        )
+        return parameter_identifier(db_name, parameter_name, object_names, alternative_name)
 
     def column_name(self, column):
         """Returns a string that concatenates the object and parameter names corresponding to the given column.
@@ -796,12 +841,11 @@ class ParameterValuePivotTableModel(PivotTableModelBase):
         for row, top_left_id in enumerate(self.model.pivot_columns):
             header_id = self.model._column_data_header[column][row]
             header_names.append(self._header_name(top_left_id, header_id))
-        return self.db_mngr.GROUP_SEP.join(header_names)
+        return DB_ITEM_SEPARATOR.join(header_names)
 
     def call_reset_model(self, pivot=None):
         """See base class."""
         object_class_ids = self._parent.current_object_class_ids
-        self._object_class_count = len(object_class_ids)
         data = self._parent.load_parameter_value_data()
         top_left_headers = [TopLeftObjectHeaderItem(self, name, id_) for name, id_ in object_class_ids.items()]
         top_left_headers += [TopLeftParameterHeaderItem(self)]
@@ -957,8 +1001,8 @@ class ParameterValuePivotTableModel(PivotTableModelBase):
         }
         if not any(db_map_entities.values()):
             return False
-        db_map_data = self._load_empty_parameter_value_data(db_map_entities=db_map_entities)
-        self.receive_data_added_or_removed(db_map_data, action)
+        data = self._load_empty_parameter_value_data(db_map_entities=db_map_entities)
+        self.receive_data_added_or_removed(data, action)
         return True
 
     def receive_relationships_added_or_removed(self, db_map_data, action):
@@ -1033,7 +1077,6 @@ class IndexExpansionPivotTableModel(ParameterValuePivotTableModel):
     def call_reset_model(self, pivot=None):
         """See base class."""
         object_class_ids = self._parent.current_object_class_ids
-        self._object_class_count = len(object_class_ids)
         data = self._parent.load_expanded_parameter_value_data()
         top_left_headers = [TopLeftObjectHeaderItem(self, name, id_) for name, id_ in object_class_ids.items()]
         self._index_top_left_header = TopLeftParameterIndexHeaderItem(self)
@@ -1105,13 +1148,22 @@ class IndexExpansionPivotTableModel(ParameterValuePivotTableModel):
 class RelationshipPivotTableModel(PivotTableModelBase):
     """A model for the pivot table in relationship input type."""
 
+    def __init__(self, parent):
+        """
+        Args:
+            parent (SpineDBEditor)
+        """
+        super().__init__(parent)
+        self._relationship_fetch_parent = _EntityFetchParent("relationship", self._parent)
+        self._object_fetch_parent = _MemberObjectFetchParent("object", self._parent)
+
     @property
     def item_type(self):
         return "relationship"
 
-    def fetch_successful(self, db_map, item):
-        entity_class_id = self._parent.current_class_id[db_map]
-        return item["class_id"] == entity_class_id
+    def _fetch_parents(self):
+        yield self._object_fetch_parent
+        yield self._relationship_fetch_parent
 
     def call_reset_model(self, pivot=None):
         """See base class."""
@@ -1205,10 +1257,25 @@ class RelationshipPivotTableModel(PivotTableModelBase):
 class ScenarioAlternativePivotTableModel(PivotTableModelBase):
     """A model for the pivot table in scenario alternative input type."""
 
+    def __init__(self, parent):
+        """
+        Args:
+            parent (SpineDBEditor)
+        """
+        super().__init__(parent)
+        self._scenario_fetch_parent = ItemTypeFetchParent("scenario")
+        self._alternative_fetch_parent = ItemTypeFetchParent("alternative")
+        self._scenario_alternative_fetch_parent = ItemTypeFetchParent("scenario_alternative")
+
     @property
     def item_type(self):
         return "scenario_alternative"
 
+    def _fetch_parents(self):
+        yield self._scenario_fetch_parent
+        yield self._alternative_fetch_parent
+        yield self._scenario_alternative_fetch_parent
+
     def call_reset_model(self, pivot=None):
         """See base class."""
         data = self._parent.load_scenario_alternative_data()
@@ -1294,12 +1361,21 @@ class ScenarioAlternativePivotTableModel(PivotTableModelBase):
 
 
 class PivotTableSortFilterProxy(QSortFilterProxyModel):
+    model_data_changed = Signal()
+
     def __init__(self, parent=None):
         """Initialize class."""
         super().__init__(parent)
         self.setDynamicSortFilter(False)  # Important so we can edit parameters in the view
         self.index_filters = {}
 
+    def setSourceModel(self, model):
+        old_model = self.sourceModel()
+        if old_model:
+            old_model.model_data_changed.disconnect(self.model_data_changed)
+        super().setSourceModel(model)
+        model.model_data_changed.connect(self.model_data_changed)
+
     def set_filter(self, identifier, filter_value):
         """Sets filter for a given index (object_class) name.
 
