@@ -24,8 +24,7 @@ from ..helpers import CharIconEngine, make_icon_background
 
 
 class ProjectItemDragMixin:
-    """Custom class with dragging support.
-    """
+    """Custom class with dragging support."""
 
     drag_about_to_start = Signal()
 
@@ -221,10 +220,9 @@ class ProjectItemSpecArray(QToolBar):
         font.setPointSize(8)
         self._button_visible.setFont(font)
         self._button_visible.setToolTip(f"<p>Show/hide {self.item_type} specifications</p>")
-        self._update_button_visible_icon_color()
         self.addWidget(self._button_visible)
         self._button_new = ShadeButton()
-        self._button_new.setIcon(QIcon(CharIconEngine("\uf067", color=Qt.darkGreen)))
+        self._button_new.setIcon(QIcon(CharIconEngine("\uf067", color=self._icon.color())))
         self._button_new.setText("New...")
         self._button_new.setToolTip(f"<p>Create new <b>{item_type}</b> specification...</p>")
         font = QFont()
@@ -240,7 +238,6 @@ class ProjectItemSpecArray(QToolBar):
         self._button_filling.setVisible(False)
         self._model.rowsInserted.connect(self._insert_specs)
         self._model.rowsAboutToBeRemoved.connect(self._remove_specs)
-        self._model.dataChanged.connect(self._change_spec_data)
         self._model.modelReset.connect(self._reset_specs)
         self._button_visible.clicked.connect(self.toggle_visibility)
         self._button_new.clicked.connect(self._show_spec_form)
@@ -248,15 +245,16 @@ class ProjectItemSpecArray(QToolBar):
 
     def set_colored_icons(self, colored):
         self._icon.set_colored(colored)
+        self.update()
+
+    def update(self):
         self._chopped_icon.update()
         self._update_button_visible_icon_color()
 
     def _update_button_visible_icon_color(self):
-        color = self._icon.color()
-        if color is not None:
-            self._button_visible.setStyleSheet(f"QToolButton{{ color: {color.name()};}}")
-        else:
-            self._button_visible.setStyleSheet("")
+        mode = QIcon.Active if self._button_base_item.isEnabled() else QIcon.Disabled
+        color = self._icon.color(mode=mode)
+        self._button_visible.setStyleSheet(f"QToolButton{{ color: {color.name()};}}")
 
     def set_color(self, color):
         bg = make_icon_background(color)
@@ -385,9 +383,9 @@ class ProjectItemSpecArray(QToolBar):
             w.set_orientation(orientation)
         style = self.style()
         extent = style.pixelMetric(style.PM_ToolBarExtensionExtent)
-        up, down, right, left = "\uf102", "\uf103", "\uf101", "\uf100"
+        down, right = "\uf0d7", "\uf0da"
         if orientation == Qt.Horizontal:
-            icon = right if not self._visible else left
+            icon = down if not self._visible else right
             width = extent
             min_width = self._button_visible.frameGeometry().right() + self._margin + spacing
             min_visible_width = min_width + self._button_new.sizeHint().width() - spacing
@@ -402,7 +400,7 @@ class ProjectItemSpecArray(QToolBar):
                 w.setMaximumWidth(w.sizeHint().width())
                 w.setMaximumHeight(height)
         else:
-            icon = down if not self._visible else up
+            icon = right if not self._visible else down
             height = extent
             min_width = self._button_new.sizeHint().height()
             min_height = self._button_visible.frameGeometry().bottom() + self._margin + spacing
@@ -441,6 +439,7 @@ class ProjectItemSpecArray(QToolBar):
             action.setVisible(self._visible)
         self._action_new.setVisible(self._visible)
 
+    @Slot(QModelIndex, int, int)
     def _insert_specs(self, parent, first, last):
         for row in range(first, last + 1):
             self._add_spec(row)
@@ -449,21 +448,18 @@ class ProjectItemSpecArray(QToolBar):
     @Slot(QModelIndex, int, int)
     def _remove_specs(self, parent, first, last):
         for row in range(first, last + 1):
-            try:
-                action = self._actions.pop(row)
-                self.removeAction(action)
-            except KeyError:
-                pass  # Happens when Plugins are removed
+            self._remove_spec(row)
         self._update_button_geom()
 
-    def _change_spec_data(self, top_left, bottom_right, roles):
-        if Qt.DisplayRole not in roles:
-            return
-        for row in range(top_left.row(), bottom_right.row() + 1):
-            index = self._model.index(row, 0)
-            button = self.widgetForAction(self._actions[row])
-            button.spec_name = index.data()
+    def _remove_spec(self, row):
+        spec_name = self._model.index(row, 0).data(Qt.DisplayRole)
+        try:
+            action = self._actions.pop(spec_name)
+            self.removeAction(action)
+        except KeyError:
+            pass  # Happens when Plugins are removed
 
+    @Slot()
     def _reset_specs(self):
         for action in self._actions.values():
             self.removeAction(action)
@@ -473,14 +469,18 @@ class ProjectItemSpecArray(QToolBar):
         self._update_button_geom()
 
     def _add_spec(self, row):
-        index = self._model.index(row, 0)
-        source_index = self._model.mapToSource(index)
-        spec = self._model.sourceModel().specification(source_index.row())
+        spec = self._model.specification(row)
         if spec.plugin:
             return
+        next_row = row + 1
+        while True:
+            next_spec = self._model.specification(next_row)
+            if next_spec is None or not next_spec.plugin:
+                break
+            next_row += 1
         button = ShadeProjectItemSpecButton(self._toolbox, spec.item_type, self._icon, spec.name)
         button.setIconSize(self.iconSize())
         button.set_orientation(self.orientation())
-        action = self.addWidget(button)
+        action = self.insertWidget(self._actions[next_spec.name], button) if next_spec else self.addWidget(button)
         action.setVisible(self._visible)
-        self._actions[row] = action
+        self._actions[spec.name] = action
